@@ -61,6 +61,12 @@ def _patch_loop(loop):
             return self._run_until_complete_orig(future)
 
     def run_once(self):
+        self._nesting_level += 1
+        if self._nesting_level == 1:
+            handle = asyncio.Handle(None, None, self)
+            handle.cancel()
+            self._bogus_handles = [handle] * len(self._ready)
+
         while self._scheduled and self._scheduled[0]._cancelled:
             self._timer_cancelled_count -= 1
             handle = heapq.heappop(self._scheduled)
@@ -94,11 +100,17 @@ def _patch_loop(loop):
             handle._run()
         handle = None
 
+        self._nesting_level -= 1
+        if self._nesting_level == 0:
+            # add bogus handles to keep loop._run_once happy
+            self._ready.extend(self._bogus_handles)
+
     cls = loop.__class__
     cls._run_until_complete_orig = cls.run_until_complete
     cls.run_until_complete = run_until_complete
-    cls._run_once = run_once
     loop._num_handles_todo = 0
+    loop._nesting_level = 0
+    loop._bogus_handles = []
 
 
 def _patch_task():
